@@ -56,7 +56,7 @@ UkwsIndicator::UkwsIndicator(QWidget *parent) : QWidget(parent)
     wmOperator = new UkwsWnckOperator;
     flowScrollArea = new QScrollArea();
     flowArea = new QWidget();
-    winboxFlowLayout = new UkwsFlowLayout(flowArea, 0, 0, 0);
+    winboxFlowLayout = new UkwsFlowLayout(flowArea, 0, 8, 8);
 
     this->setContentsMargins(0, 0, 0, 0);
     flowArea->setLayout(winboxFlowLayout);
@@ -251,16 +251,21 @@ void UkwsIndicator::reloadWindowList(int boxMinHeight)
     wmOperator->updateWindowList();
     int size = wmOperator->windowQList->size();
 
-    /*
-     * 计算indicator大小、window box大小
-     * switch模式：
-     *   1. indicator宽高都限定为3/4屏，即上下左右各留3/8的空白区间
-     *   2. winbox与indicator间距22px（来自windows）
-     *   3. 不需要在indicator中显示当前选定的窗口名称
-     *
-     * 平铺模式：
-     *     对高度进行确定，两行高度计算是否能满足，不行则用三行高度来算
-     */
+    // 获取屏幕大小
+    QDesktopWidget *desktop = QApplication::desktop();
+    int screenNum = desktop->screenNumber(this);
+    QRect screenRect = QGuiApplication::screens().at(screenNum)->geometry();
+
+    // 计算侧边栏工作区控件大小
+    float scale = (float)config->workspaceItemUnits / config->workspaceAllUnits;
+
+    // 计算侧边栏工作区缩略图大小
+    int w = screenRect.width() * scale - 20;
+    int h = screenRect.height() * w / screenRect.width();
+
+    // 计算winbox拖拽缩略图大小
+    QSize dragIconSize = QSize(w * 0.8, h * 0.8);
+
 
     for (int i = 0; i < size; i++) {
         WnckWindow *win = wmOperator->windowQList->at(i);
@@ -271,6 +276,7 @@ void UkwsIndicator::reloadWindowList(int boxMinHeight)
         // 设置Winbox大小
         wb->setOrigThumbnailByWnck();
         wb->setWinboxSizeByHeight(boxMinHeight);
+        wb->setDragIconSize(dragIconSize);
 
 //        winboxList.append(wb);
         wb->setIconByWnck();
@@ -299,7 +305,7 @@ void UkwsIndicator::reSetWindowThumbnailByWnck()
     updateDesktopViewRequestId++;
     for (i = 0; i < cpus; i++) {
         workThread = new QThread;
-        worker = new UkwsWorker;
+        worker = new UkwsWorker(UkwsWorker::Winbox);
 
         // doingThread只是保存索引，无父子关系
         worker->doingThread = workThread;
@@ -316,7 +322,7 @@ void UkwsIndicator::reSetWindowThumbnailByWnck()
     int size = winboxList.size();
     for (i = 0; i < size; i++) {
         wb = winboxList.at(i);
-        workerList[i % cpus]->workList.append(wb);
+        workerList[i % cpus]->appedWorkItem(wb);
     }
 
     // 开始处理
@@ -350,15 +356,23 @@ void UkwsIndicator::reShow(UkwsIndicatorShowMode mode, int minScale)
 //    QDesktopWidget *desktop = QApplication::desktop();
 
     // 自当前屏幕上显示indicator
-    // 获取鼠标所在屏幕
+
     QRect screenRect;
     int screenCount = QGuiApplication::screens().count();
     int screenIndex = -1;
-    for (int i = 0  ; i < screenCount; i++) {
-//        screenRect = desktop->screenGeometry(i);
-        screenRect = QGuiApplication::screens().at(i)->geometry() ;
-        if (screenRect.contains(QCursor::pos()))
-            screenIndex = i;
+
+    if (showMode == UkwsIndicatorShowMode::ShowModeSwitch) {
+        // 获取鼠标所在屏幕
+        for (int i = 0  ; i < screenCount; i++) {
+            //        screenRect = desktop->screenGeometry(i);
+            screenRect = QGuiApplication::screens().at(i)->geometry() ;
+            if (screenRect.contains(QCursor::pos()))
+                screenIndex = i;
+        }
+    } else {
+        // 多任务视图展现在主屏上
+        QDesktopWidget *desktop = QApplication::desktop();
+        screenIndex = desktop->primaryScreen();
     }
 
 //    screenRect =  desktop->availableGeometry(screenIndex);
@@ -370,7 +384,8 @@ void UkwsIndicator::reShow(UkwsIndicatorShowMode mode, int minScale)
     } else {
         maxWidth = screenRect.width();
         maxHeight = screenRect.height();
-        maxWidth = maxWidth * 5 / 6 - maxWidth * 1 / 20;
+//        maxWidth = maxWidth * 5 / 6 - maxWidth * 1 / 20;
+        maxWidth = maxWidth * config->workspacePrimaryAreaUnits / config->workspaceAllUnits;
         maxHeight = maxHeight;
     }
     winBoxHeight = ((maxHeight - 5 - 5 - 5 - 5) / scale) - 0 - 32 - 5 - 5;
@@ -379,6 +394,8 @@ void UkwsIndicator::reShow(UkwsIndicatorShowMode mode, int minScale)
     mainLayout->setMargin(20);
 
     // 默认以(maxWidth, maxHeight)为大小，以(maxHeight / 5)为winBox高度
+//    winboxFlowLayout->setSizeConstraint(QLayout::SetMaximumSize);
+    winboxFlowLayout->setGeometry(QRect(0, 0, maxWidth, maxHeight));
     this->resize(maxWidth, maxHeight);
     this->reloadWindowList(winBoxHeight);
 
@@ -396,15 +413,19 @@ void UkwsIndicator::reShow(UkwsIndicatorShowMode mode, int minScale)
         return;
     }
 
-    for (int i = (minScale * 2 - 1); i > 2; i--) {
+    for (int i = (minScale * 2 - 1); i > 4; i--) {
         scale = i / 2.0;
         winBoxHeight = ((maxHeight - 5 - 5 - 5 - 5) / scale) - 0 - 32 - 5 - 5;
 
         foreach (UkwsWindowBox *wb, winboxList) {
             wb->setWinboxSizeByHeight(winBoxHeight);
+            wb->show();
         }
 
-        minSize = this->getMaxRect(QRect(0, 0, maxWidth, maxHeight));
+//        minSize = this->getMaxRect(QRect(0, 0, maxWidth, maxHeight));
+        winboxFlowLayout->setGeometry(QRect(0, 0, maxWidth, maxHeight));
+        minSize = QSize(winboxFlowLayout->maxWidth, winboxFlowLayout->maxHeight);
+
         ratio = (float)minSize.width() / minSize.height();
 
         if ((minSize.height() < maxHeight) && (ratio >= 4 / 3)) {
@@ -413,11 +434,16 @@ void UkwsIndicator::reShow(UkwsIndicatorShowMode mode, int minScale)
             break;
     }
 
+    if (showMode == UkwsIndicatorShowMode::ShowModeTiling)
+        actualScale = 2.5;
+
     winBoxHeight = ((maxHeight - 5 - 5 - 5 - 5) / actualScale) - 0 - 32 - 5 - 5;
     foreach (UkwsWindowBox *wb, winboxList) {
         wb->setWinboxSizeByHeight(winBoxHeight);
     }
-    minSize = this->getMaxRect(QRect(0, 0, maxWidth, maxHeight));
+//    minSize = this->getMaxRect(QRect(0, 0, maxWidth, maxHeight));
+    winboxFlowLayout->setGeometry(QRect(0, 0, maxWidth, maxHeight));
+    minSize = QSize(winboxFlowLayout->maxWidth, winboxFlowLayout->maxHeight);
 
     if (showMode == UkwsIndicatorShowMode::ShowModeSwitch) {
         winboxFlowLayout->setSizeConstraint(QLayout::SetMinimumSize);
@@ -426,8 +452,8 @@ void UkwsIndicator::reShow(UkwsIndicatorShowMode mode, int minScale)
         if (minSize.height() + 20 * 2 + 14 < maxHeight)
             maxHeight =  minSize.height() + 20 * 2 + 14;
         this->setFixedSize(maxWidth, maxHeight);
-        this->move((screenRect.width() - maxWidth) / 2,
-                   (screenRect.height() - maxHeight) / 2);
+        this->move((screenRect.width() - maxWidth) / 2 + screenRect.left(),
+                   (screenRect.height() - maxHeight) / 2 + screenRect.top());
     }
     this->show();
     flowScrollBar->hide();
@@ -673,10 +699,15 @@ bool UkwsIndicator::eventFilter(QObject *object, QEvent *event)
         }
 
         if (event->type() == QEvent::MouseButtonRelease) {
-            QMouseEvent *mouseEvent = static_cast<QMouseEvent *>(event);
-            if (mouseEvent->button() == Qt::LeftButton)
-                dragWindow = false;
-            return true;
+            if (dragWindow) {
+                QMouseEvent *mouseEvent = static_cast<QMouseEvent *>(event);
+                if (mouseEvent->button() == Qt::LeftButton)
+                    dragWindow = false;
+
+                return true;
+            }
+
+            return false;
         }
 
         if (event->type() == QEvent::MouseMove) {
@@ -711,15 +742,12 @@ bool UkwsIndicator::eventFilter(QObject *object, QEvent *event)
             int wbIndex;
 
             dataStream >> indIndex >> wbIndex >> wbTitle;
-            qDebug() << "===== dragEnter Indicator:" << indIndex << wbIndex << wbTitle;
-
             dragEvent->accept();
         }
 
         if (event->type() == QEvent::DragLeave) {
             QDragLeaveEvent *dragLeave = static_cast<QDragLeaveEvent *>(event);
 
-            qDebug() << "===== DragLeave Indicator:" << index;
             dragLeave->accept();
         }
 
@@ -733,7 +761,6 @@ bool UkwsIndicator::eventFilter(QObject *object, QEvent *event)
             int wbIndex;
 
             dataStream >> indIndex >> wbIndex >> wbTitle;
-            qDebug() << "===== Window Drop," << indIndex << wbIndex << wbTitle;
             if (indIndex != index) {
 //                emit windowChangeWorkspace(wbIndex, indIndex, index);
 //                dropEvent->accept();
@@ -755,8 +782,11 @@ bool UkwsIndicator::eventFilter(QObject *object, QEvent *event)
 
         if (event->type() == QEvent::Enter) {
             if ((winboxFlowLayout->visualHeight > winboxFlowLayout->geometry().height())
-                    && (flowScrollBar->maximum() > 0))
+                    && (flowScrollBar->maximum() > 0)
+                    && (this->showMode == UkwsIndicatorShowMode::ShowModeTiling))
                 flowScrollBar->show();
+            else
+                flowScrollBar->hide();
             QWidget::enterEvent(event);
 
             return false;
