@@ -36,6 +36,8 @@ using namespace UkwsHelperXHeader;
 
 #include <QX11Info>
 
+#define NO_ADD_FRAME
+
 static const QSize UKWS_TITLE_SIZE = QSize(UKWS_TITLE_WIDTH, UKWS_TITLE_HEIGHT);
 static const QSize UKWS_ICON_SIZE = QSize(UKWS_ICON_DEFAULT_WIDTH, UKWS_ICON_DEFAULT_HEIGHT);
 static const QSize UKWS_THUMBNAIL_SIZE = QSize(UKWS_THUMBNAIL_DEFAULT_WIDTH, UKWS_THUMBNAIL_DEFAULT_HEIGHT);
@@ -50,6 +52,7 @@ UkwsWindowBox::UkwsWindowBox(QWidget *parent) : QWidget(parent)
     winBottomOffset = 0;
     frameXid = 0;
     hasFrame = false;
+    drag = nullptr;
 
     titleLabel = new UkwsWindowExtraLabel();
     iconLabel = new UkwsWindowExtraLabel();
@@ -58,6 +61,7 @@ UkwsWindowBox::UkwsWindowBox(QWidget *parent) : QWidget(parent)
     titleSize = UKWS_TITLE_SIZE;
     iconSize = UKWS_ICON_SIZE;
     winboxSize = QSize(UKWS_WINDOWBOX_WIDTH, UKWS_WINDOWBOX_HEIGHT);
+    dragIconSize = QSize(0, 0);
 
     iconLabel->setAlignment(Qt::AlignCenter);
     thumbnailLabel->setAlignment(Qt::AlignCenter);
@@ -110,9 +114,9 @@ UkwsWindowBox::UkwsWindowBox(QWidget *parent) : QWidget(parent)
      *
      */
 
-    mainLayout->setSpacing(UKWS_WIDGET_SPACING);
+//    mainLayout->setSpacing(UKWS_WIDGET_SPACING);
     mainLayout->setAlignment(Qt::AlignTop | Qt::AlignHCenter);
-    topBarLayout->setSpacing(UKWS_WIDGET_SPACING);
+//    topBarLayout->setSpacing(UKWS_WIDGET_SPACING);
     topBarLayout->setAlignment(Qt::AlignTop | Qt::AlignLeft);
 
     topBarLayout->addWidget(iconLabel);
@@ -130,11 +134,19 @@ UkwsWindowBox::UkwsWindowBox(QWidget *parent) : QWidget(parent)
     thumbnailLabel->setObjectName(UKWS_OBJ_WINBOX_THUMBNAIL);
     this->setObjectName(UKWS_OBJ_WINBOX);
 
-    mainLayout->setContentsMargins(5, 5, 5, 5);
+    // 对齐设置暂时不生效，使用下面的方式修正
+//    mainLayout->setContentsMargins(8, 0, 8, 8);
+    mainLayout->setMargin(8);
     setThumbnailNormal();
 
 //    connect(this, &UkwsWindowBox::clicked, this, &UkwsWindowBox::activateWnckWindow);
 
+    scaleTimes = 0;
+    scaleTimer.setSingleShot(true);
+    scaleTimer.setTimerType(Qt::CoarseTimer);
+    scaleTimer.setInterval(UKWS_DRAG_SCALE_INTERVAL_MS);
+    connect(&scaleTimer, &QTimer::timeout, this, &UkwsWindowBox::scaleDragPixmap);
+
     this->setAttribute(Qt::WA_TranslucentBackground, true);
     titleLabel->setAttribute(Qt::WA_TranslucentBackground, true);
     iconLabel->setAttribute(Qt::WA_TranslucentBackground, true);
@@ -154,29 +166,28 @@ void UkwsWindowBox::paintEvent(QPaintEvent *event)
     QWidget::paintEvent(event);
 }
 
-void UkwsWindowBox::setSubWidgetSize(int w, int h)
+void UkwsWindowBox::setSubWidgetSizeByThnSize(int w, int h)
 {
-    // winbox宽w，高h，外框2；
-    // icon宽高32，上下间距0，左间距5（缩略图的左边框2，保持对齐），边框0；
-    // title左边距0，右边距5（缩略图右边框2，保持对齐），边框0；
-    // 调整值：2x2 + 5 + 32 + 5 = 46
-    titleLabel->setFixedSize(w - 46, 32);
+    // icon，宽高32，上下间距0，上下边框0，右边距5；
+    // 缩略图控件，外边距8，外边框4，边框紧贴图片，宽占用：(8 + 4) * 2 = 24，高占用：8 + 4 = 12；
+    // 高度调整值：8 + 32 + 16 = 56；
+    // 宽度调整值：32 + 5 = 37
+    // 标题宽度调整值：
+    titleLabel->setFixedSize(w - 37, 32);
     this->updateTitleBySize();
 
-    // winbox宽w，高h，外框2；
-    // icon宽高32，上下间距0
-    // 缩略图，外边距3；
-    // 调整值：宽，2x2 + 3x2 = 10，高，2x2 + 32 + 3x2 = 42
-    thumbnailLabel->setFixedSize(w - 10, h - 42);
+    // 缩略图控件，外边框4，边框紧贴图片，宽、高占用：4 * 2 = 8
+    thumbnailLabel->setFixedSize(w + 8, h + 8);
 }
 
 void UkwsWindowBox::setWinboxSizeByHeight(int height)
 {
-    // winbox边框2；
-    // icon，高32，上下间距0，上下边框0；
-    // 缩略图，外边距3，外边框2，图片间隔2；
-    // 调整值：高，2x2 + 32 + 3x2 + 2x2 + 2x2 = 50
-    int thnHeight = height - 50;
+    // winbox边框4，内边距4，宽、高占用：（4 + 4） * 2 = 16；
+    // icon，高32，上下间距0，上下边框0，宽、高占用：32；
+    // 缩略图控件，外边距8，外边框4，边框紧贴图片，宽占用：(8 + 4) * 2 = 24，高占用：8 + 4 = 12；
+    // 高度调整值：16 + 32 + 12 = 60；
+    // 宽度调整值：24
+    int thnHeight = height - 60;
 
     int w, h;
     float scale;
@@ -199,12 +210,8 @@ void UkwsWindowBox::setWinboxSizeByHeight(int height)
     w = int(w * scale);
     h = int(h * scale);
 
-    // winbox边框2；
-    // icon高32，上下间距0，上下边框0；
-    // 缩略图宽w高h，外边距3，外边框2，图片间隔2；
-    // 调整值：宽，2x2 + 3x2 + 2x2 + 2x2 = 18，高，2x2 + 32 + 3x2 + 2x2 + 2x2 = 50
-    this->setSubWidgetSize(w + 18, h + 48);
-    this->setFixedSize(w + 18, height);
+    this->setSubWidgetSizeByThnSize(w, h);
+    this->setFixedSize(w + 24, height);
 }
 
 WnckWindow *UkwsWindowBox::getWnckWindow()
@@ -269,6 +276,14 @@ void UkwsWindowBox::fixFrameWindowArea()
     winWidth = frameWidth;
     winHeight = frameHeight;
 
+#ifdef NO_ADD_FRAME
+    winLeftOffset = 0;
+    winRightOffset = 0;
+    winTopOffset = 0;
+    winBottomOffset = 0;
+    frameXid = wnck_window_get_xid(wnckWin);
+    hasFrame = false;
+#else
     if ((origWidth == frameWidth) && (origHeight == frameHeight)) {
         // 无窗口装饰区
         winLeftOffset = 0;
@@ -319,6 +334,7 @@ void UkwsWindowBox::fixFrameWindowArea()
 
         hasFrame = true;
     }
+#endif
 }
 
 void UkwsWindowBox::setOrigThumbnailByWnck()
@@ -336,8 +352,7 @@ void UkwsWindowBox::setThumbnail(QPixmap origPixmap)
     QSize labelSize = thumbnailLabel->size();
     thumbnailLabel->originalQPixmap = origPixmap;
 
-    // 缩略图，外边框2，图片间距2；
-    // 调整值：宽，2x2 + 2x2 = 8，高，2x2 + 2x2 = 8
+    // 缩略图控件包含4px的外边框，需要减去外边框的大小
     scaledThumbnail = origPixmap.scaled(labelSize.width() - 8,
                                         labelSize.height() - 8,
                                         Qt::KeepAspectRatio,
@@ -345,7 +360,8 @@ void UkwsWindowBox::setThumbnail(QPixmap origPixmap)
 
     QSize thumbnailSize = scaledThumbnail.size();
     QSize size = (thumbnailLabel->contentsRect().size() - thumbnailSize) / 2;
-    thumbnailOffset = QPoint(size.width(), size.height());
+    thumbnailOffset = QPoint(size.width() + 4, size.height() + 4);
+    scaledThumbnail = makeRadiusPixmap(scaledThumbnail, 6);
     thumbnailLabel->setPixmap(scaledThumbnail);
 }
 
@@ -360,8 +376,7 @@ void UkwsWindowBox::setThumbnailByWnck()
                                                                         winTopOffset,
                                                                         winBottomOffset);
 
-    // 缩略图，外边框2，图片间距2；
-    // 调整值：宽，2x2 + 2x2 = 8，高，2x2 + 2x2 = 8
+    // 缩略图控件包含4px的外边框，需要减去外边框的大小
     scaledThumbnail = thumbnailLabel->originalQPixmap.scaled(labelSize.width() - 8,
                                                              labelSize.height() - 8,
                                                              Qt::KeepAspectRatio,
@@ -369,10 +384,16 @@ void UkwsWindowBox::setThumbnailByWnck()
 
     QSize thumbnailSize = scaledThumbnail.size();
     QSize size = (thumbnailLabel->contentsRect().size() - thumbnailSize) / 2;
-    thumbnailOffset = QPoint(size.width(), size.height());
+    thumbnailOffset = QPoint(size.width() + 4, size.height() + 4);
+    scaledThumbnail = makeRadiusPixmap(scaledThumbnail, 6);
     thumbnailLabel->setPixmap(scaledThumbnail);
 }
 
+void UkwsWindowBox::setDragIconSize(QSize size)
+{
+    dragIconSize = size;
+}
+
 QPixmap UkwsWindowBox::windowPixmap()
 {
     return thumbnailLabel->originalQPixmap;
@@ -395,12 +416,16 @@ void UkwsWindowBox::setThumbnailNormal()
 
 void UkwsWindowBox::setWindowBoxSelected()
 {
-    this->setStyleSheet("QWidget#winbox{padding:0px; border:2px solid rgb(255, 255, 255, 255);}");
+    this->setStyleSheet("QWidget#winbox{padding:0px;"
+                        "border:4px solid rgb(255, 255, 255, 127);"
+                        "border-radius: 6px;}");
 }
 
 void UkwsWindowBox::setWindowBoxUnselected()
 {
-    this->setStyleSheet("QWidget#winbox{padding:0px; border:2px solid rgb(255, 255, 255, 0);}");
+    this->setStyleSheet("QWidget#winbox{padding:0px;"
+                        "border:4px solid rgb(255, 255, 255, 0);"
+                        "border-radius: 6px;}");
 }
 
 void UkwsWindowBox::moveToWorkspace(int wsIndex)
@@ -410,6 +435,54 @@ void UkwsWindowBox::moveToWorkspace(int wsIndex)
     wnck_window_move_to_workspace(wnckWin, workspace);
 }
 
+// 构建圆角图片
+QPixmap UkwsWindowBox::makeRadiusPixmap(QPixmap orig, int radius)
+{
+    // 构造圆角裁剪区域
+    QRectF rect = QRectF(0, 0, orig.width(), orig.height());
+    QPainterPath path;
+    path.addRoundedRect(rect, radius, radius);
+
+    // 设置新全透明背景图片
+    QPixmap dest(orig.size());
+    dest.fill(Qt::transparent);
+
+    // 裁剪圆角区域，将原图贴到裁剪好的区域，构建圆角图片
+    QPainter painter(&dest);
+    painter.setClipPath(path);
+    painter.drawPixmap(orig.rect(), orig);
+
+    return dest;
+}
+
+void UkwsWindowBox::scaleDragPixmap()
+{
+    if (scaleTimes < 0) {
+        return;
+    }
+
+    if (scaleTimes == 0) {
+        return;
+    }
+
+    if (drag == nullptr) {
+        return;
+    }
+
+    scaleTimes--;
+    scaleTimer.start();
+
+    // 设置缩放大小
+    QSize size = scaledThumbnail.size() - scaleUnitSize * (UKWS_DRAG_SCALE_TIMES - scaleTimes);
+    QPixmap pixmap = scaledThumbnail.scaled(size, Qt::KeepAspectRatio,
+                                            Qt::FastTransformation);
+    drag->setPixmap(pixmap);
+    update();
+//    drag->setHotSpot(mouseEvent->pos() - thumbnailOffset);
+
+
+}
+
 bool UkwsWindowBox::eventFilter(QObject *watched, QEvent *event)
 {
     static bool pressed = 0;
@@ -463,12 +536,12 @@ bool UkwsWindowBox::eventFilter(QObject *watched, QEvent *event)
             QPixmap tempPixmap = scaledThumbnail;
             QPainter painter;
             painter.begin(&tempPixmap);
-            painter.fillRect(scaledThumbnail.rect(), QColor(0, 0, 0, 127));
+            painter.fillRect(scaledThumbnail.rect(), QColor(0, 0, 0, 159));
             painter.end();
             thumbnailLabel->setPixmap(tempPixmap);
 
             // 拖拽处理
-            QDrag *drag = new QDrag(this);
+            drag = new QDrag(this);
             QMimeData *mimeData = new QMimeData;
             QByteArray byteData;
             QDataStream dataStream(&byteData, QIODevice::WriteOnly);
@@ -476,27 +549,30 @@ bool UkwsWindowBox::eventFilter(QObject *watched, QEvent *event)
             dataStream << parentIndex << index << title;
             mimeData->setData("application/x-dnditemdata", byteData);
             drag->setMimeData(mimeData);
-            drag->setPixmap(scaledThumbnail);
-//            drag->setHotSpot(mouseEvent->pos() - winbox->pos() + QPoint(0 - 14, 0 + 32 - 14));
+//            drag->setPixmap(scaledThumbnail);
+            drag->setPixmap(scaledThumbnail.scaled(dragIconSize, Qt::KeepAspectRatio,
+                                                          Qt::FastTransformation));
 
-            qDebug() << "----------------";
-            qDebug() << thumbnailLabel->size() - scaledThumbnail.size();
-            qDebug() << thumbnailLabel->contentsRect().size() - scaledThumbnail.size();
-            qDebug() << thumbnailOffset;
-            qDebug() << thumbnailLabel->size();
-            qDebug() << thumbnailLabel->contentsRect().size();
+            // 事件发生时的坐标，减去thn边框的宽度，获得实际图片上的坐标
+            QPoint hotSpot = (mouseEvent->pos() - thumbnailOffset) *
+                    (float)drag->pixmap().size().width() / (float)scaledThumbnail.width();
+            drag->setHotSpot(hotSpot);
 
-            drag->setHotSpot(mouseEvent->pos() - thumbnailOffset);
+            // 设置缩放大小
+            scaleUnitSize = (scaledThumbnail.size() - drag->pixmap().size()) / UKWS_DRAG_SCALE_TIMES;
+
+            // 设置拖拽图片缩放动画的定时器
+            scaleTimer.start();
+            scaleTimes = UKWS_DRAG_SCALE_TIMES;
 
             if (drag->exec()) {
-                qDebug() << "Drag Done";
                 thumbnailLabel->setPixmap(scaledThumbnail);
             } else {
-                qDebug() << "Drag Cancel";
                 thumbnailLabel->setPixmap(scaledThumbnail);
             }
 
             drag->deleteLater();
+            drag = nullptr;
 
             return true;
         }
